//+------------------------------------------------------------------+
//|                                           NonLinearKalman_v1.mq4 |
//|                           Copyright  2007, TrendLaboratory Ltd. |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, TrendLaboratory Ltd."
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"
//---- indicator settings
#property indicator_separate_window 
#property indicator_buffers 5
#property indicator_color1 LimeGreen
#property indicator_color2 LimeGreen
#property indicator_color3 PaleVioletRed
#property indicator_color4 PaleVioletRed
#property indicator_color5 Silver 
#property indicator_width1 2
#property indicator_width3 2
#property indicator_width5 2


//---- indicator parameters
extern int     Price            =  0;  //Price mode : 0-Close,1-Open,2-High,3-Low,4-Median,5-Typical,6-Weighted
extern int     Order            =  3;  //Filter Order: 1-EMA,2-2nd Order,3-3rd Order
extern int     Length           = 14;  //Fast Filter Period 
extern int     PreSmooth        =  3;  //Pre-smoothing period
extern int     PreSmoothMode    =  3;  //Pre-smoothing MA Mode: 0-SMA,1-EMA,2-SMMA,3-LWMA
extern int     ZmPeriod         = 10;
extern int     ZmMaMethod       = MODE_SMA;
extern int     NormPeriod       = 25;
extern bool    UseZeroMean      = false;
extern bool    UseNormalization = true;

//---- indicator buffers
double     ZMKalman[];
double     Kalman[];
double     LowPass[];
double     Delta[];
double     work[];
double     slope[];
double     histUu[];
double     histUd[];
double     histDd[];
double     histDu[];

int        draw_begin;
double     a, b, c, pi = 3.1415926535;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- indicator buffers mapping
   IndicatorBuffers(8);
   SetIndexBuffer(0,histUu); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,histUd); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,histDd); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,histDu); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,ZMKalman);
   SetIndexBuffer(5,LowPass);
   SetIndexBuffer(6,Delta);
   SetIndexBuffer(7,Kalman);
//---- drawing settings
   draw_begin = 2*(Length+3) + PreSmooth;
   SetIndexDrawBegin(0,draw_begin);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("NonLinearKalman("+Order+","+Length+","+PreSmooth+")");
   SetIndexLabel(0,"NinLinearKalman");
//----
   if(Order == 1)
   a = 2.0/(1+Length);
   else
   if(Order == 2)
   {
   a = MathExp(-MathSqrt(2)*pi/Length);
	b = 2*a*MathCos(MathSqrt(2)*pi/Length);
   }
   else
   if(Order == 3)
   {
   a = MathExp(-pi/Length);
	b = 2*a*MathCos(MathSqrt(3)*pi/Length);
   c = MathExp(-2*pi/Length);
   } 
//---- initialization done
   return(0);
}
//+------------------------------------------------------------------+
//| NonLinearKalman_v1                                                 |
//+------------------------------------------------------------------+
int start()
{
   int limit, i,r, shift;
   int counted_bars=IndicatorCounted();
   double Smoother, Detrend;
      if (ArraySize(work)!=Bars)  ArrayResize(work,Bars);
      if (ArraySize(slope)!=Bars) ArrayResize(slope,Bars);
   
   if(counted_bars<1)
   for(i=1;i<=draw_begin;i++) 
   {
   Kalman[Bars-i]=iMA(NULL,0,1,0,0,Price,Bars-i); 
   LowPass[Bars-i]=iMA(NULL,0,1,0,0,Price,Bars-i); 
   Delta[Bars-i]=0; 
   }
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;

   for(shift=limit; shift>=0; shift--)
   {
   Smoother = iMA(NULL,0,PreSmooth,0,PreSmoothMode,Price,shift);
      
      if(Order == 1) 
      {
      LowPass[shift] = (1-a)*LowPass[shift+1] + a*Smoother;
      Detrend = Smoother - LowPass[shift]; 
      Delta[shift] =  (1-a)*Delta[shift+1] + a*Detrend;
      }
      else
		if(Order == 2) 
		{
		LowPass[shift] = b*LowPass[shift+1] - a*a*LowPass[shift+2] + (1 - b + a*a)*Smoother;
		Detrend = Smoother - LowPass[shift]; 
		Delta[shift] = b*Delta[shift+1] - a*a*Delta[shift+2] + (1 - b + a*a)*Detrend;
		}
		else
		if(Order == 3) 
		{
		LowPass[shift] = (b+c)*LowPass[shift+1] - (c+b*c)*LowPass[shift+2] + c*c*LowPass[shift+3] + (1-b+c)*(1-c)*Smoother;
		Detrend = Smoother - LowPass[shift];
		Delta[shift] = (b+c)*Delta[shift+1] - (c+b*c)*Delta[shift+2] + c*c*Delta[shift+3] + (1-b+c)*(1-c)*Detrend;
		}
	Kalman[shift] = LowPass[shift] + Delta[shift]; 	
   }
   
   //
   //
   //
   //
   //
   //
   
   for(shift=limit, r=Bars-shift-1; shift>=0; shift--,r++)
   {
      double displace   = 0;
      double multiplier = 1;
      
         //
         //
         //
         //
         //
         
         if (UseZeroMean)
         {
            double mean = iMAOnArray(Kalman,0,ZmPeriod,0,ZmMaMethod,shift);
            double dev  = iDeviation(Kalman,ZmPeriod,mean,shift);
            if (dev!=0)
                  work[r] = (Kalman[shift]-mean)/dev;
            else  work[r] = 0;
         }
         else work[r] = Kalman[shift];
         if (UseNormalization)
         {
            double hh = work[r];
            double ll = work[r];
               for (int k=1; k<NormPeriod; k++) 
               {
                  hh = MathMax(work[r-k],hh);
                  ll = MathMin(work[r-k],ll);
               }
            if (!UseZeroMean) { displace = -0.5; multiplier = 2.0; }
            if (hh!=ll)
                  ZMKalman[shift] = multiplier*((work[r]-ll)/(hh-ll)+displace);
            else  ZMKalman[shift] = 0;
         }
         else ZMKalman[shift] = work[r];
         
         //
         //
         //
         //
         //
         
         if (UseZeroMean || UseNormalization)
         {
            slope[r] = slope[r-1];
               if (ZMKalman[shift]>ZMKalman[shift+1]) slope[r] =  1;
               if (ZMKalman[shift]<ZMKalman[shift+1]) slope[r] = -1;
                  histUu[shift] = EMPTY_VALUE;
                  histUd[shift] = EMPTY_VALUE;
                  histDd[shift] = EMPTY_VALUE;
                  histDu[shift] = EMPTY_VALUE;
                  if (ZMKalman[shift]>0)
                     if (slope[r]==1)
                           histUu[shift] = ZMKalman[shift];
                     else  histUd[shift] = ZMKalman[shift];
                  if (ZMKalman[shift]<0)
                     if (slope[r]==-1)
                           histDd[shift] = ZMKalman[shift];
                     else  histDu[shift] = ZMKalman[shift];
         }
   }         
//---- done
   return(0);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double iDeviation(double& array[], double period, double ma, int i, bool isSample=true)
{
   double sum = 0.00; for(int k=0; k<period; k++) sum += MathPow((array[i+k]-ma),2);
   if (isSample)      
         return(MathSqrt(sum/(period-1.0)));
   else  return(MathSqrt(sum/period));
}